@@ -3,13 +3,14 @@
 # database module
 #-----------------------------
 
+import copy 
 import json
 import sqlite3
 import multiprocessing
 from typing import Any
-from ctypes import Union
 from . import lib, cursor, search
 
+
 def _to_json(data:dict) -> str:
     """
     Convert data to json string
@@ -24,7 +25,7 @@ def _to_json(data:dict) -> str:
     return json.dumps(data)
 
 
-def _from_json(data) -> dict:
+def _from_json(data:str) -> dict:
     """
     Convert data to json string
 
@@ -152,19 +153,66 @@ class Database(object):
 
 class Document(dict):
     """
-    ::Document
+    :: Document
     """
 
+    # Private columns
+    PRIVATE_COLUMNS = ["_id", "_json", "_created_at", "_modified_at"]
+
     def __init__(self, collection, row: sqlite3.Row):
         self.collection = collection 
         self._load(row)
+
+    def dget(self, path:str, default=None)->Any:
+        """
+        Return a property by DotNotation
+
+        ie: 
+            #dget("key.deep1.deep2.deep3")
+
+        Args:
+            path:str - the dotnotation path
+            default:Any - default value 
         
-    def __getattr__(self, name):
-        return self[name]
+        Returns:
+            Any
+        """
+        return lib.dict_get(obj=dict(self), path=path, default=default)
+
+    def dset(self, path:str, value:Any):
+        """
+        Set a property by DotNotation
+
+        Args:
+            path:str - the dotnotation path
+            value:Any - The value
+
+        Returns:
+            Void
+        """
+        data = copy.deepcopy(dict(self))
+        lib.dict_set(obj=data, path=path, value=value)
+        self.update(data)
+
+    def dpop(self, path:str):
+        """ 
+        Remove a property by DotNotation and return the value
+
+        Args:
+            path:str
+
+        Returns:
+            Any: the value that was removed
+        """
+        data = copy.deepcopy(dict(self))
+        v = lib.dict_pop(obj=data, path=path)
+        self.update(data)
+        return v
 
     def update(self, *a, **kw):
         """
         Update the active Document
+
         ie:
             #update(key=value, key2=value2, ...)
             #update({ "key": value, "key2": value2 })
@@ -176,15 +224,17 @@ class Document(dict):
         Returns:
             Document
         """
-        _ = {}
+        data = {}
         if a and isinstance(a[0], dict):
-            _.update(a[0])
-        _.update(kw)
-        self._load(self.collection.update(_id=self._id, doc=_, _as_document=False))
+            data.update(a[0])
+        data.update(kw)
+
+        row = self.collection.update(_id=self._id, doc=data, _as_document=False)
+        self._load(row)
 
     def delete(self):
         """
-        Delete the Doument
+        Delete the Doument from the collection
 
         Returns:
             None
@@ -192,6 +242,16 @@ class Document(dict):
         self.collection.delete(self._id)
         self._empty_self()
 
+    def commit(self):
+        """
+        To commit the data when it's mutated outside.
+            doc = Document()
+            doc["xone"][1] = True
+            doc.commit()
+        """
+        data = dict(self)
+        self.update(data)
+
     def _load(self, row:sqlite3.Row):
         """
         load the content into the document
@@ -199,12 +259,15 @@ class Document(dict):
         Args:
             row: sqlite3.Row
         """
+
         self._empty_self()
-        self._id = row["_id"]
-        doc = {}
-        if "_json" in row:
-            doc = {**_from_json(row["_json"])}
-        doc.update({k: row[k] for k in row.keys()})
+        row = {k: row[k] for k in row.keys()}
+        _json = _from_json(row.pop("_json"))
+        self._id = row.get("_id")
+        doc = {
+            **_json,
+            **row # row columns takes precendece
+        }
         super().__init__(doc)
 
     def _empty_self(self):
@@ -265,7 +328,7 @@ class Collection(object):
         """
         with self.db:
             try:
-                _exec = self.db.execute("SELECT * FROM %s WHERE type=?" % "sqlite_master", ("index",))
+                _exec = self.db.execute("SELECT * FROM sqlite_master WHERE type=?", ("index",))
                 return [item["tbl_name"] for item in _exec]
             except: pass
         return []
@@ -281,7 +344,6 @@ class Collection(object):
         with self.db:
             _exec = self.db.execute("SELECT count(*) FROM %s;" % self.name)
             return _exec.fetchone()[0]
-        return 0
 
     # ---- methods ----
 
@@ -291,7 +353,7 @@ class Collection(object):
 
         Args:
           _id:str - the document id
-          _as_document:bool - when True it will return the raw format of the data
+          _as_document:bool - when True return Document
           **kw other query
 
         Returns:
@@ -299,7 +361,7 @@ class Collection(object):
 
         """
 
-        _as_document = False
+        _as_document = True
         if "_as_document" in kw:
             _as_document = kw.pop("_as_document")
 
@@ -335,7 +397,6 @@ class Collection(object):
 
         query += " LIMIT 1"
 
-        xquery = " AND ".join(["%s %s" % k for k in smart_filters])
         xparams = list(filters.values())
 
         with self.db:
@@ -343,7 +404,7 @@ class Collection(object):
             row = _exec.fetchone()
             if row:
                 return Document(self, row) if _as_document else row
-            return None
+        return None
 
     def insert(self, doc: dict, _as_document:bool=True) -> Document:
         """
@@ -388,7 +449,8 @@ class Collection(object):
         Args:
           _id:str - document id
           doc:dict - the document to update
-          replace:bool - to completely replace the document
+          replace:bool - By default document will be merged with existing data
+                  When True, it will save it as is. 
 
         Returns:
             Document
@@ -398,7 +460,6 @@ class Collection(object):
             if rdoc:
                 _doc = doc if replace else lib.dict_merge(_from_json(rdoc["_json"]), doc)
                 ts = lib.get_timestamp()
-                q = "UPDATE %s SET _json=?, _modified_at=? WHERE _id=?" % self.name
                 
                 if "_id" in _doc:
                     del doc["_id"]
@@ -423,20 +484,15 @@ class Collection(object):
                 return self.get(_id=_id, _as_document=_as_document)
             return None
 
-    def replace(self, _id:str, doc:dict, _as_document=True) -> Document:
-        """
-        To replace a document with a new document
-        """
-        return self.update(_id=_id, doc=doc, replace=True, _as_document=_as_document)
-
-    def unset(self, _id: str, attrs: list = []):
-        """
-        To remove atrributes
-        """
-
     def delete(self, _id: str) -> bool:
         """
         To delete an entry by _id
+        
+        Args:
+            _id:str - entry id
+
+        Returns:
+            Bool
         """
         with self.db:
             self.db.execute("DELETE FROM %s WHERE _id=?" % (self.name), (_id, ))
